<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
{% load panya_inclusion_tags %}

<html>
<head>
	<title>{{ object.title }}</title>
</head>

<body>
    {% render_object object.as_leaf_class 'peek' %}
</body>
</html>
