{% load likes_inclusion_tags %}

<div class="heading">
    {% likes object.modelbase_obj %}
	<h1>{{ object.title }}</h1>
	<p class="meta">By <a href="{{ object.owner.get_absolute_url }}">{{ object.owner }}</a> on {{ object.created|date:"j F Y" }}</p>
    {% comment %}
	<p class="social">
	    <a href="javascript: void(0);" class="social_share">Share</a>
		<a href="#contribute" class="social_comments">Comments</a>
		<a href="javascript: void(0);" class="social_like">I Like</a>
	</p>
	<p class="social_pop">
	    <input type="text" name="share" value="http://www.radio2000.co.za/" />
		<a href="#" class="social_mail"></a>
		<a href="#" class="social_face"></a>
		<a href="#" class="social_twit"></a>
	</p>
    {% endcomment %}
</div><!--/heading-->
