{% extends "base.html" %}
{% load panya_inclusion_tags %} 

{% block content %}
    <div class="page section_news">
	    <div class="title">
		    <h1>{{ title }}</h1>
		</div><!--/title-->

        {% view_modifier view_modifier %}

        {% modelbase_listing object_list "list_wide" %}

        {% pager page_obj %}

	</div><!--/page-->
{% endblock %}
