{% extends "base.html" %}
{% load panya_inclusion_tags gallery_inclusion_tags %} 

{% block content %}
    <div class="page section_news">
	    <div class="title">
		    <h1>{{ title }}</h1>
		</div><!--/title-->
            
        {% view_modifier view_modifier %}

        {% object_header object %}
      
        {% render_object object.as_leaf_class "detail" %}

        {% object_comments object %}

	</div><!--/page-->
{% endblock %}
